searchData={"items":[{"type":"module","title":"Prototype","doc":"Documentation for `Prototype`.","ref":"Prototype.html"},{"type":"function","title":"Prototype.apply/3","doc":"Calls functions by searching up a prototype chain of modules, calling it on\nthe first module where it is found.","ref":"Prototype.html#apply/3"},{"type":"function","title":"Prototype.call/2","doc":"","ref":"Prototype.html#call/2"},{"type":"function","title":"Prototype.function_chain/3","doc":"Finds all matching functions up the prototype chain and returns them as a list.\n\n(Note that a function of the correct arity may be exported by a module,\nbut a `FunctionClauseError` would still be raised if there is not a matching\nfunction *clause* in that module. However, a matching clause may be found up\nfurther the prototype chain.)","ref":"Prototype.html#function_chain/3"},{"type":"function","title":"Prototype.has_function?/3","doc":"Returns whether the given module exports the given function.","ref":"Prototype.html#has_function?/3"},{"type":"function","title":"Prototype.has_prototype?/1","doc":"Returns the prototype module for a given module, if one exists.","ref":"Prototype.html#has_prototype?/1"},{"type":"function","title":"Prototype.prototype_chain/1","doc":"Recursively builds a list of all modules in the prototype chain.\nWorks for both modules and structs.","ref":"Prototype.html#prototype_chain/1"},{"type":"extras","title":"Prototype","doc":"# Prototype\n\nAn Elixir library that implements prototype inheritance for dynamic function calling\nover a chain of modules. These prototypes can be user-defined struct modules\nor any other modules.","ref":"readme.html"},{"type":"extras","title":"Usage - Prototype","doc":"To specify a module's prototype module, implement the `__prototype__/0` function like so:\n\n```Elixir\ndefmodule YourModule do\n  def __prototype__, do: AnyOtherModule\nend\n```\n\nThen to call functions wherever they are found up the prototype chain\nyou might:\n\n```Elixir\ncase Prototype.apply(YourModule, :function_name, [\"argument\", \"list\"]) do\n  {:ok, return_value, _meta} -> return_value,\n  {:missing_fun, _function_name, _args, _meta} -> {:error, \"function not found\"}\nend\n```\n\nWhere `_meta` contains tracing metadata about the call chain that executed.\nAs you might suspect, if the function cannot be found in any of the modules\nthen the result is a :missing_fun tuple because it's not very fun to have\nmultiple attempts to call a function fail!","ref":"readme.html#usage"},{"type":"extras","title":"Installation - Prototype","doc":"This package can be installed by adding `prototype` to\nyour list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:prototype, \"~> 1.0.0-alpha\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Feedback welcome! - Prototype","doc":"If you use this library and have suggestions for how to make it better,\nplease [file an issue](https://github.com/typesend/prototype/issues).","ref":"readme.html#feedback-welcome"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}